vim.opt.number = true
vim.opt.relativenumber = true
vim.opt.expandtab = true
vim.opt.tabstop = 8
vim.opt.softtabstop = 0
vim.opt.shiftwidth = 4
vim.opt.smarttab = true
vim.opt.autoindent = true
vim.opt.mouse = "a"

vim.cmd("colorscheme habamax")

local plugins = {
    {
        source = "neovim/nvim-lspconfig",
    },
    {
        source = "hrsh7th/nvim-cmp",
        depends = { "hrsh7th/cmp-nvim-lsp" }
    },
    {
        source = "andweeb/presence.nvim"
    }
}

local servers = {
    clangd = {}
}

-- Install plugin manager
local mini_path = vim.fn.stdpath("data") .. "/site/pack/deps/start/mini.nvim"
if not vim.loop.fs_stat(mini_path) then
    vim.fn.system({
        "git", "clone", "--filter=blob:none",
        "https://github.com/echasnovski/mini.nvim", mini_path
    })

    vim.cmd("packadd mini.nvim | helptags ALL")
end
require("mini.deps").setup({ path = { package = path_package } })

require("mini.sessions").setup({
    autoread = true,
    autowrite = true
})

-- <C-_> is Ctrl+/
local comment_keybind = "<C-_>"
require("mini.comment").setup({
    mappings = {
        comment = comment_keybind,
        comment_line = comment_keybind,
        comment_visual = comment_keybind,
        textobject = comment_keybind,
    },
})

-- Install plugins
for plugin in pairs(plugins) do
    MiniDeps.add(plugins[plugin])
end

-- Setup completion
local cmp = require("cmp")
cmp.setup({
    snippet = {
        expand = function(args)
            vim.snippet.expand(args.body)
        end,
    },
    mapping = cmp.mapping.preset.insert({
        ["<C-Space>"] = cmp.mapping.complete(),
        ["<CR>"] = cmp.mapping.confirm({ select = true })
    }),
    sources = cmp.config.sources({
        { name = "nvim_lsp" },
    })
}, {
    { name = "buffer" },
});

-- Setup servers and their configs
local capabilities = require("cmp_nvim_lsp").default_capabilities()
local lspconfig = require("lspconfig")
for server in pairs(servers) do
    local config = servers[server]
    config.capabilities = config.capabilities or capabilities

    lspconfig[server].setup(config)
end

-- Setup Discord rich presence
require("presence").setup({
    neovim_image_text = "Neovim",
})

-- Binds to jump between snippets
vim.keymap.set({ "i", "s" }, "<Tab>", function()
    if vim.snippet.active({ direction = 1 }) then
        return "<Cmd>lua vim.snippet.jump(1)<CR>"
    else
        return "<Tab>"
    end
end, { expr = true })

vim.keymap.set({ "i", "s" }, "<S-Tab>", function()
    if vim.snippet.active({ direction = -1 }) then
        return "<Cmd>lua vim.snippet.jump(-1)<CR>"
    else
        return "<S-Tab>"
    end
end, { expr = true })
